@@ -15,7 +15,7 @@
  *
  * The Initial Developer of the Original Code is
  * Josh Geenen <gcontactsync@pirules.org>.
- * Portions created by the Initial Developer are Copyright (C) 2009-2010
+ * Portions created by the Initial Developer are Copyright (C) 2009-2015
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
@@ -40,7 +40,8 @@ if (!com.gContactSync) {com.gContactSync = {};}
 
 window.addEventListener("load",
   /** Initializes the Accounts class when the window has finished loading */
-  function gCS_AccountsLoadListener(e) {
+  function gCS_AccountsLoadListener() {
+    window.removeEventListener("load", gCS_AccountsLoadListener, false);
     com.gContactSync.Accounts.initDialog();
   },
 false);
@@ -85,63 +86,6 @@ com.gContactSync.Accounts = {
       com.gContactSync.LOGGER.LOG_WARNING("Error in Accounts.initDialog", e);
     }
   },
-  /**
-   * Create a new username/account for the selected plugin.
-   * @returns {boolean} True if an authentication HTTP request was sent.
-   */
-  newUsername: function Accounts_newUsername() {
-    var prompt   = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
-                             .getService(Components.interfaces.nsIPromptService)
-                             .promptUsernameAndPassword,
-        username = {},
-        password = {},
-        // opens a username/password prompt
-        ok = prompt(window, com.gContactSync.StringBundle.getStr("loginTitle"),
-                    com.gContactSync.StringBundle.getStr("loginText"), username, password, null,
-                    {value: false});
-    if (!ok) {
-      return false;
-    }
-    if (com.gContactSync.LoginManager.getAuthToken(username.value)) { // the username already exists
-      com.gContactSync.alertWarning(com.gContactSync.StringBundle.getStr("usernameExists"));
-      return false;
-    }
-    // This is a primitive way of validating an e-mail address, but Google takes
-    // care of the rest.  It seems to allow getting an auth token w/ only the
-    // username, but returns an error when trying to do anything w/ that token
-    // so this makes sure it is a full e-mail address.
-    if (username.value.indexOf("@") < 1) {
-      com.gContactSync.alertError(com.gContactSync.StringBundle.getStr("invalidEmail"));
-      return this.newUsername();
-    }
-    // fix the username before authenticating
-    username.value = com.gContactSync.fixUsername(username.value);
-    var body    = com.gContactSync.gdata.makeAuthBody(username.value, password.value),
-        httpReq = new com.gContactSync.GHttpRequest("authenticate", null, null, body);
-    // if it succeeds and Google returns the auth token, store it and then start
-    // a new sync
-    httpReq.mOnSuccess = function newUsernameSuccess(httpReq) {
-      com.gContactSync.LoginManager.addAuthToken(username.value,
-                                                 'GoogleLogin' + httpReq.responseText.split("\n")[2]);
-      com.gContactSync.Accounts.selectedAbChange();
-      com.gContactSync.Accounts.fillUsernames();
-    };
-    // if it fails, alert the user and prompt them to try again
-    httpReq.mOnError   = function newUsernameError(httpReq) {
-      com.gContactSync.alertError(com.gContactSync.StringBundle.getStr('authErr'));
-      com.gContactSync.LOGGER.LOG_ERROR('Authentication Error - ' +
-                                        httpReq.status,
-                                        httpReq.responseText);
-      com.gContactSync.Accounts.newUsername();
-    };
-    // if the user is offline, alert them and quit
-    httpReq.mOnOffline = function newUsernameOffline(httpReq) {
-      com.gContactSync.alertWarning(com.gContactSync.StringBundle.getStr('offlineErr'));
-      com.gContactSync.LOGGER.LOG_ERROR(com.gContactSync.StringBundle.getStr('offlineErr'));
-    };
-    httpReq.send();
-    return true;
-  },
   /**
    * Returns the GAddressBook corresponding to the currently-selected address
    * book in the accounts tree.
@@ -415,26 +359,59 @@ com.gContactSync.Accounts = {
    */
   getAllGroups: function Accounts_getAllGroups(aUsername) {
     this.restoreGroups();
-    if (!aUsername || aUsername === "none")
+    if (!aUsername || aUsername === "none") {
       return false;
+    }
     var token = com.gContactSync.LoginManager.getAuthTokens()[aUsername];
     if (!token) {
       com.gContactSync.LOGGER.LOG_WARNING("Unable to find the token for username " + aUsername);
       return false;
     }
+    this.getAccessToken(token, aUsername);
+    return true;
+  },
+  /**
+   * Exchanges the refresh token for an access token.  On success, continues synchronization.
+   *
+   * @param aRefreshToken {string} A refresh token.
+   */
+  getAccessToken: function Accounts_getAccessToken(aRefreshToken, aUsername) {
+    com.gContactSync.LOGGER.VERBOSE_LOG("Requesting access token");
+    // Fetch an access token from the refresh token
+    var request = new com.gContactSync.GHttpRequest("REFRESH_REQUEST", aRefreshToken);
+    request.mOnSuccess = function getAccessTokenSuccess(aHttpRequest) {
+      var response = JSON.parse(aHttpRequest.responseText);
+      var accessToken = response.token_type + " " + response.access_token;
+      com.gContactSync.Accounts.sendGetGroupsRequest(accessToken, aUsername);
+    };
+    request.mOnError = function getAccessTokenError(httpReq) {
+      com.gContactSync.LOGGER.LOG_WARNING(httpReq.responseText);
+    };
+    request.mOnOffline = null;
+    request.send();
+  },
+  /**
+   * Sends the request to get the groups for a given username.
+   *
+   * @param aAccessToken {string} The access token.
+   * @param aUsername {string The username.
+   */
+  sendGetGroupsRequest: function Accounts_finishGetAllGroups(aAccessToken, aUsername) {
     com.gContactSync.LOGGER.VERBOSE_LOG("Fetching groups for username: " + aUsername);
-    var httpReq = new com.gContactSync.GHttpRequest("getGroups", token, null,
-                                   null, aUsername);
+    var httpReq = new com.gContactSync.GHttpRequest("getGroups",
+                                                    aAccessToken,
+                                                    null,
+                                                    null,
+                                                    aUsername);
     httpReq.mOnSuccess = function getAllGroupsSuccess(httpReq) {
       com.gContactSync.LOGGER.VERBOSE_LOG(com.gContactSync.serializeFromText(httpReq.responseText));
       com.gContactSync.Accounts.addGroups(httpReq.responseXML, aUsername);
     };
     httpReq.mOnError   = function getAllGroupsError(httpReq) {
-      com.gContactSync.LOGGER.LOG_ERROR(httpReq.responseText);
+      com.gContactSync.LOGGER.LOG_WARNING(httpReq.responseText);
     };
     httpReq.mOnOffline = null;
     httpReq.send();
-    return true;
   },
   /**
    * Adds groups in the given atom feed to the Groups menulist provided the
