@@ -15,7 +15,7 @@
  *
  * The Initial Developer of the Original Code is
  * Josh Geenen <gcontactsync@pirules.org>.
- * Portions created by the Initial Developer are Copyright (C) 2008-2012
+ * Portions created by the Initial Developer are Copyright (C) 2008-2015
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
@@ -34,13 +34,14 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-if (!com) var com = {}; // A generic wrapper variable
+if (!com) {var com = {};} // A generic wrapper variable
 // A wrapper for all GCS functions and variables
-if (!com.gContactSync) com.gContactSync = {};
+if (!com.gContactSync) {com.gContactSync = {};}
 
 window.addEventListener("load",
   /** Initializes the CardDialogOverlay when the window has finished loading. */
-  function gCS_CardDialogOverlayLoadListener(e) {
+  function gCS_CardDialogOverlayLoadListener() {
+    window.removeEventListener("load", gCS_CardDialogOverlayLoadListener, false);
     com.gContactSync.CardDialogOverlay.init();
   },
 false);
@@ -78,6 +79,8 @@ com.gContactSync.CardDialogOverlay = {
   mLoadNumber: 0,
   /** This stores whether the contact is read-only (ie from LDAP or the Mac AB) */
   mDisabled:   false,
+  /** Whether the application is Postbox */
+  mIsPostbox:  false,
   /**
    * Adds a tab to the tab box, if possible.  Waits until the abCardOverlay is
    * loaded.
@@ -118,8 +121,9 @@ com.gContactSync.CardDialogOverlay = {
     }
     // add the email type drop down menus
     try {
+      this.mIsPostbox = this.addPostboxEmailType(document.getElementById("PrimaryEmail"));
       // add the type for Postbox (this does nothing for TB or Seamonkey)
-      if (!this.addPostboxEmailType(document.getElementById("PrimaryEmail"))) {
+      if (!this.mIsPostbox) {
         var emailTypes       = com.gContactSync.gdata.contacts.EMAIL_TYPES,
             primaryEmailBox  = this.getBox("PrimaryEmail"),
             secondEmailBox   = this.getBox("SecondEmail"),
@@ -138,6 +142,11 @@ com.gContactSync.CardDialogOverlay = {
     catch (ex0) {
       com.gContactSync.alertError("Unable to setup email types: " + ex0);
     }
+    // Hide the gContactSync tab for Postbox
+    // TODO - implement full support for Postbox instead.
+    if (this.mIsPostbox) {
+      document.getElementById("gContactSyncTab").collapsed = true;
+    }
     var newDialog      = false, // post-Mailnews Core Bug 63941 - TODO - can this be removed (Postbox?)
         showPhoneTypes = com.gContactSync.Preferences.mSyncPrefs.phoneTypes.value,
         swap           = com.gContactSync.Preferences.mSyncPrefs.swapMobilePager.value,
@@ -334,7 +343,13 @@ com.gContactSync.CardDialogOverlay = {
     anniversaryElem._constrainValue = birthdayElem._constrainValue;
     anniversaryElem._setFieldValue = birthdayElem._setFieldValue;
 
-    // fix the width of the dialog
+    // Set the height of the Notes field
+    var notesElem = document.getElementById("Notes");
+    if (notesElem && com.gContactSync.Preferences.mSyncPrefs.notesHeight.value) {
+      notesElem.style.height = com.gContactSync.Preferences.mSyncPrefs.notesHeight.value;
+    }
+
+    // Update the size of the dialog
     window.sizeToContent();
 
     // override the check and set card values function
