@@ -15,7 +15,7 @@
  *
  * The Initial Developer of the Original Code is
  * Josh Geenen <gcontactsync@pirules.org>.
- * Portions created by the Initial Developer are Copyright (C) 2008-2014
+ * Portions created by the Initial Developer are Copyright (C) 2008-2015
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
@@ -34,13 +34,14 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-if (!com) var com = {}; // A generic wrapper variable
+if (!com) {var com = {};} // A generic wrapper variable
 // A wrapper for all GCS functions and variables
-if (!com.gContactSync) com.gContactSync = {};
+if (!com.gContactSync) {com.gContactSync = {};}
 
 window.addEventListener("load",
   /** Initializes the ContactConverter class when the window has finished loading */
-  function gCS_ContactConverterLoadListener(e) {
+  function gCS_ContactConverterLoadListener() {
+    window.removeEventListener("load", gCS_ContactConverterLoadListener, false);
     com.gContactSync.ContactConverter.init();
   },
 false);
@@ -227,81 +228,11 @@ com.gContactSync.ContactConverter = {
       aGContact.setValue(obj.elementName, obj.index, type, value);
       nonEmpty = nonEmpty || value;
     }
-    // Birthday can be either YYYY-M-D or --M-D for no year.
-    // TB can have all three, just a day/month, or just a year through the UI
-    var birthDay    = parseInt(aTBContact.getValue("BirthDay"), 10),
-        birthMonth  = (isNaN(birthDay) || (birthDay > 31)) ?
-                      null : aTBContact.getValue("BirthMonth"),
-        birthdayVal = null;
-    // if the contact has a birth month (and birth day) add it to the contact
-    // from Google
-    if (birthMonth && !isNaN(parseInt(birthMonth, 10)) && (birthMonth <= 12)) {
-      var birthYear = parseInt(aTBContact.getValue("BirthYear"), 10);
-      // if the birth year is NaN or 0, use '-'
-      if (!birthYear) {
-        birthYear = "-";
-      }
-      // otherwise pad it to 4 characters
-      else {
-        birthYear = String(birthYear);
-        while (birthYear.length < 4) {
-          birthYear = "0" + birthYear;
-        }
-      }
-      // Pad the birth month to 2 characters
-      birthMonth = String(birthMonth);
-      while (birthMonth.length < 2) {
-        birthMonth = "0" + birthMonth;
-      }
-      // Pad the birth day to 2 characters
-      birthDay = String(birthDay);
-      while (birthDay.length < 2) {
-        birthDay = "0" + birthDay;
-      }
-      // form the birthday string: year-month-day
-      birthdayVal = birthYear + "-" + birthMonth + "-" + birthDay;
-    }
-    com.gContactSync.LOGGER.VERBOSE_LOG(" * Birthday: " + birthdayVal);
-    nonEmpty = nonEmpty || birthdayVal;
-    aGContact.setValue("birthday", 0, null, birthdayVal);
 
-    var anniversaryDay = parseInt(aTBContact.getValue("AnniversaryDay"), 10);
-    var anniversaryMonth = (isNaN(anniversaryDay) || anniversaryDay > 31) ?
-                             null :
-                             parseInt(aTBContact.getValue("AnniversaryMonth"), 10);
-    var anniversaryYear = (isNaN(anniversaryMonth) || anniversaryMonth > 13) ?
-                            null :
-                            parseInt(aTBContact.getValue("AnniversaryYear"), 10);
-    anniversaryDay = String(anniversaryDay);
-    anniversaryMonth = String(anniversaryMonth);
-    while (anniversaryDay.length < 2) {
-      anniversaryDay = "0" + anniversaryDay;
-    }
-    while (anniversaryMonth.length < 2) {
-      anniversaryMonth = "0" + anniversaryMonth;
-    }
-    var anniversaryVal = anniversaryYear ? anniversaryYear + "-" + anniversaryMonth + "-" + anniversaryDay : null;
-    com.gContactSync.LOGGER.VERBOSE_LOG(" * Anniversary: " + anniversaryVal);
-    nonEmpty = nonEmpty || anniversaryVal;
-    aGContact.setValue("event", 0, "anniversary", anniversaryVal);
+    nonEmpty = this.setGoogleBirthday(aGContact, aTBContact) || nonEmpty;
+    nonEmpty = this.setGoogleAnniversary(aGContact, aTBContact) || nonEmpty;
+    nonEmpty = this.setExtendedProperties(aGContact, aTBContact) || nonEmpty;
 
-    // set the extended properties
-    aGContact.removeExtendedProperties();
-    arr = com.gContactSync.Preferences.mExtendedProperties;
-    var props = {};
-    for (i = 0, length = arr.length; i < length; i++) {
-      // add this extended property if it isn't a duplicate or blank
-      if (arr[i] && !props[arr[i]]) {
-        props[arr[i]] = true;
-        value = this.checkValue(aTBContact.getValue(arr[i]));
-        nonEmpty = nonEmpty || value;
-        aGContact.setExtendedProperty(arr[i], value);
-      }
-      else if (arr[i] != "") {
-        com.gContactSync.LOGGER.LOG_WARNING("Found a duplicate extended property: " +
-                                            arr[i]);
-      }
-    }
     // If the myContacts pref is set and this contact is new then add the
     // myContactsName group
     // Group membership does not make a contact not "empty" according to Google's API.
@@ -311,7 +242,7 @@ com.gContactSync.ContactConverter = {
       }
     // If syncing all groups then find all the lists this contact is in and set
     // those as the contact's groups
-    } else if (ab.mPrefs.syncGroups == "true") {
+    } else if (ab.mPrefs.syncGroups === "true") {
       // set the groups
       var groups = [],
           list;
@@ -332,7 +263,7 @@ com.gContactSync.ContactConverter = {
     // Upload the photo
     // Photos do not make a contact non-empty by Google's API.
     if (com.gContactSync.Preferences.mSyncPrefs.sendPhotos.value) {
-      aGContact = this.savePhotoFromTBContact(aTBContact, aGContact);
+      this.savePhotoFromTBContact(aTBContact, aGContact);
     }
     
     // Add the phonetic first and last names
@@ -362,7 +293,6 @@ com.gContactSync.ContactConverter = {
    *                            Components.interfaces.nsIAbMDBCard if this is
    *                            before 413260 landed.
    * @param aGContact {GContact} A GContact object to update with the TB contact's photo.
-   * @returns {GContact} The updated GContact.
    */
   savePhotoFromTBContact: function ContactConverter_savePhotoFromTBContact(aTBContact, aGContact) {
     // Get the profile directory
@@ -386,7 +316,6 @@ com.gContactSync.ContactConverter = {
     } else {
       aGContact.setPhoto("");
     }
-    return aGContact;
   },
   /**
    * Converts an GContact's Atom/XML representation of a contact to
@@ -440,11 +369,286 @@ com.gContactSync.ContactConverter = {
     }
     
     // Get the birthday info
-    var bday = aGContact.getValue("birthday", 0),
-        year  = null,
+    this.setTBBirthday(aTBContact, aGContact.getValue("birthday", 0));
+    this.setTBAnniversary(aTBContact, aGContact.getValue("event", 0, "anniversary"));
+
+    if (com.gContactSync.Preferences.mSyncPrefs.getPhotos.value) {
+
+      this.savePhotoFromGContact(aTBContact, aGContact);
+    }
+    
+    // Add the phonetic first and last names
+    if (com.gContactSync.Preferences.mSyncPrefs.syncPhoneticNames.value) {
+      aTBContact.setValue("PhoneticFirstName",
+                          aGContact.getAttribute("givenName",
+                                                 com.gContactSync.gdata.namespaces.GD.url,
+                                                 0,
+                                                 "yomi"));
+      aTBContact.setValue("PhoneticLastName",
+                          aGContact.getAttribute("familyName",
+                                                 com.gContactSync.gdata.namespaces.GD.url,
+                                                 0,
+                                                 "yomi"));
+    }
+
+    aTBContact.update();
+    if (ab.mPrefs.syncGroups == "true" && ab.mPrefs.myContacts != "true") {
+      this.setTBGroups(aTBContact, aGContact);
+    }
+  },
+  /**
+   * Sets the group (mailing list) membership for the given TB contact.
+   * @param aTBContact {TBContact} The Thunderbird contact to update.
+   * @param aGContact {GContact} The Google contact to get the groups from.
+   */
+  setTBGroups: function ContactConverter_setTBGroups(aTBContact, aGContact) {
+    var groups = aGContact.getValue("groupMembershipInfo"),
+        lists  = com.gContactSync.Sync.mLists,
+        list,
+        group;
+    for (var i in lists) {
+      group = groups[i];
+      list  = lists[i];
+      // delete the card from the list, if necessary
+      if (list.hasContact(aTBContact)) {
+        if (!group) {
+          list.deleteContacts([aTBContact]);
+        }
+        aTBContact.update();
+      }
+      // add the card to the list, if necessary
+      else if (group) {
+        list.addContact(aTBContact);
+      }
+    }
+  },
+  /**
+   * Merges the two contacts together.  This is a very primitive merge and only
+   * copies values from both sides into the other.
+   * If both contacts have a value the updateGoogleInConflicts pref decides which
+   * value "wins".
+   * @param aTBContact {TBContact} The Thunderbird contact to merge.
+   * @param aGContact {GContact} The Google contact to merge.
+   * @param aUpdateGoogleInConflicts {boolean} Whether Google should be updated during conflicts.
+   * @return Whether the Google contact was updated.
+   */
+  merge: function ContactConverter_merge(aTBContact, aGContact, aUpdateGoogleInConflicts) {
+
+    if (!this.mInitialized)
+      this.init();
+
+    this.mCurrentCard = aTBContact;
+    var ab = aTBContact.mAddressBook;
+
+    aTBContact.setValue("GoogleID", aGContact.id);
+    aTBContact.setValue("LastModifiedDate", 0);
+
+    // Shift phone numbers to avoid duplicates.
+    var numberAttributes = ["WorkPhone", "HomePhone", "FaxNumber", "CellularNumber", "PagerNumber", "HomeFaxNumber", "OtherNumber"];
+    var numbers = [];
+    for (var attr in numberAttributes) {
+      var tbValue = aTBContact.getValue(numberAttributes[attr]);
+      if (tbValue) {numbers.push(new com.gContactSync.Property(tbValue, aTBContact.getValue(numberAttributes[attr] + "Type")));}
+    }
+    for (var attr in numberAttributes) {
+      var prop = numbers.length ? numbers.shift : new com.gContactSync.Property(null, null);
+      aTBContact.setValue(numberAttributes[attr], prop.value);
+      aTBContact.setValue(numberAttributes[attr] + "Type", prop.type);
+    }
+
+    var gContactUpdated = false;
+    var tbContactUpdated = false;
+    var arr = this.mConverterArr;
+
+    for (var i = 0, length = arr.length; i < length; i++) {
+
+      var obj = arr[i],
+          property = aGContact.getValue(obj.elementName, obj.index, obj.type),
+          value = this.checkValue(aTBContact.getValue(obj.tbName)),
+          type = aTBContact.getValue(obj.tbName + "Type") || obj.type;
+
+      property = property || new com.gContactSync.Property("", "");
+
+      if (obj.tbName === com.gContactSync.dummyEmailName &&
+          com.gContactSync.isDummyEmail(value)) {
+        value = null;
+        type  = null;
+      }
+
+      com.gContactSync.LOGGER.VERBOSE_LOG(obj.tbName + ": '" + property.value +
+                                          "'/'" + value + "' , type: '" + property.type +
+                                          "'/'" + type + "'");
+
+      // If TB has a value and (Google's is empty or update Google in conflict) update Google
+      // Else if Google has a value update TB
+      if (value != property.value || type != property.type) {
+        if (value && (!property.value || aUpdateGoogleInConflicts)) {
+          gContactUpdated = true;
+          aGContact.setValue(obj.elementName, obj.index, type, value);
+        } else if (property.value) {
+          tbContactUpdated = true;
+          aTBContact.setValue(obj.tbName, property.value);
+          aTBContact.setValue(obj.tbName + "Type", property.type);
+          if (property.type) {aTBContact.setValue(obj.tbName + "Type", property.type);}
+        }
+      }
+    }
+
+    // When merging take the extended properties from TB.  No other add-ons use these.
+    if (this.setExtendedProperties(aGContact, aTBContact)) {
+      // TODO - determine if there were actual changes
+      gContactUpdated = true;
+    }
+
+    // Merge photos
+    var tbPhoto = aTBContact.getValue("PhotoName");
+    var gPhoto = aGContact.getPhotoInfo().etag;
+    if (tbPhoto && (!gPhoto || aUpdateGoogleInConflicts)) {
+      if (com.gContactSync.Preferences.mSyncPrefs.sendPhotos.value) {
+        this.savePhotoFromTBContact(aTBContact, aGContact);
+      }
+    } else if (gPhoto) {
+      if (com.gContactSync.Preferences.mSyncPrefs.getPhotos.value) {
+        tbContactUpdated = true;
+        this.savePhotoFromGContact(aTBContact, aGContact);
+      }
+    }
+
+    // Merge birthday
+    var tbBirthday = aTBContact.getValue("BirthMonth");
+    var gBirthday = aGContact.getValue("birthday");
+    if (tbBirthday && (!gBirthday || aUpdateGoogleInConflicts)) {
+      var oldValue = gBirthday ? gBirthday.value : null;
+      gContactUpdated = (this.setGoogleBirthday(aGContact, aTBContact) != oldValue) || gContactUpdated;
+    } else if (gBirthday) {
+      tbContactUpdated = true;
+      this.setTBBirthday(aTBContact, gBirthday);
+    }
+
+    // Merge anniversary
+    var tbAnniversary = aTBContact.getValue("AnniversaryMonth");
+    var gAnniversary = aGContact.getValue("event", 0, "anniversary");
+    if (tbAnniversary && (!gAnniversary || aUpdateGoogleInConflicts)) {
+      var oldValue = gAnniversary ? gAnniversary.value : null;
+      gContactUpdated = (this.setGoogleAnniversary(aGContact, aTBContact) != oldValue) || gContactUpdated;
+    } else if (gAnniversary) {
+      tbContactUpdated = true;
+      this.setTBAnniversary(aTBContact, gAnniversary);
+    }
+
+    var tbPhonFirst = aTBContact.getValue("PhoneticFirstName");
+    var gPhonFirst  = aGContact.getAttribute("givenName",
+                                             com.gContactSync.gdata.namespaces.GD.url,
+                                             0,
+                                             "yomi");
+    if (tbPhonFirst != gPhonFirst) {
+      if (tbPhonFirst && (!gPhonFirst || aUpdateGoogleInConflicts)) {
+        gContactUpdated = true;
+        aGContact.setAttribute("givenName",
+                               com.gContactSync.gdata.namespaces.GD.url,
+                               0,
+                               "yomi",
+                               tbPhonFirst);
+      } else if (gPhonFirst) {
+        tbContactUpdated = true;
+        aTBContact.setValue("PhoneticFirstName", gPhonFirst);
+      }
+    }
+
+    var tbPhonLast = aTBContact.getValue("PhoneticLastName");
+    var gPhonLast  = aGContact.getAttribute("familyName",
+                                             com.gContactSync.gdata.namespaces.GD.url,
+                                             0,
+                                             "yomi");
+    if (tbPhonLast != gPhonLast) {
+      if (tbPhonLast && (!gPhonLast || aUpdateGoogleInConflicts)) {
+        gContactUpdated = true;
+        aGContact.setAttribute("familyName",
+                               com.gContactSync.gdata.namespaces.GD.url,
+                               0,
+                               "yomi",
+                               tbPhonLast);
+      } else if (gPhonLast) {
+        tbContactUpdated = true;
+        aTBContact.setValue("PhoneticLastName", gPhonLast);
+      }
+    }
+
+    // If synchronizing groups Google wins since it always has groups.  Not all add-ons sync groups.
+    if (ab.mPrefs.syncGroups == "true" && ab.mPrefs.myContacts != "true") {
+      if (tbContactUpdated) {aTBContact.update();}  // Must update the contact before changing mailing lists.
+      tbContactUpdated = true;
+      this.setTBGroups(aTBContact, aGContact);
+    }
+
+    if (tbContactUpdated) {
+      aTBContact.update();
+    }
+    return {google: gContactUpdated, thunderbird: tbContactUpdated};
+  },
+  /**
+   * Saves the photo from the given Google contact to the given TB contact if present and if it has changed
+   * since the last sync.
+   * @param aGContact {GContact} A GContact object with the photo to save.
+   * @param aTBContact {TBContact}   An existing card that can be QI'd to
+   *                            Components.interfaces.nsIAbMDBCard if this is
+   *                            before 413260 landed.  Updated with the photo from the GContact.
+   */
+  savePhotoFromGContact: function ContactConvert_savePhotoFromGContact(aTBContact, aGContact) {
+
+    aTBContact.updatePhoto(aGContact.getPhotoInfo());
+  },
+  /**
+   * Check if the given string is null, of length 0, or consists only of spaces
+   * and return null if any of the listed conditions is true.
+   * This function was added to fix Bug 20389: Values with only spaces should be
+   * treated as empty
+   * @param aValue {string} The string to check.
+   * @returns null   - The string is null, of length 0, or consists only of
+                      spaces
+   *         aValue - The string has at least one character that is not a space
+   */
+  checkValue: function ContactConverter_checkValue(aValue) {
+    if (!aValue || !aValue.length) return null;
+    for (var i = 0; i < aValue.length; i++)
+      if (aValue[i] != " ") return aValue;
+    return null;
+  },
+  /**
+   * Sets the extended properties in the given Google contact from the given TB contact.
+   *
+   * @param aGContact {GContact} The Google contact to update.
+   * @param aTBContact {TBContact} The Thunderbird contact.
+   * @return {boolean} Whether the contact has any extended properties.
+   */
+  setExtendedProperties: function ContactConverter_setExtendedProperties(aGContact, aTBContact) {
+    aGContact.removeExtendedProperties();
+    arr = com.gContactSync.Preferences.mExtendedProperties;
+    var props = {};
+    var nonEmpty = false;
+    for (i = 0, length = arr.length; i < length; i++) {
+      // add this extended property if it isn't a duplicate or blank
+      if (arr[i] && !props[arr[i]]) {
+        props[arr[i]] = true;
+        value = this.checkValue(aTBContact.getValue(arr[i]));
+        aGContact.setExtendedProperty(arr[i], value);
+      }
+      else if (arr[i] != "") {
+        com.gContactSync.LOGGER.LOG_WARNING("Found a duplicate extended property: " +
+                                            arr[i]);
+      }
+    }
+    return nonEmpty;
+  },
+  /**
+   * Sets the birthday of the given contact.
+   * @param aTBContact {TBContact} The Thunderbird contact.
+   * @param bday {string} The birthday string.  null if not present.
+   */
+  setTBBirthday: function ContactConverter_setTBBirthday(aTBContact, bday) {
+    var year  = null,
         month = null,
         day   = null;
-    // If it has a birthday...
     if (bday && bday.value) {
       com.gContactSync.LOGGER.VERBOSE_LOG(" * Found a birthday value of " + bday.value);
       // If it consists of all three date elements: YYYY-M-D
@@ -467,9 +671,13 @@ com.gContactSync.ContactConverter = {
     aTBContact.setValue("BirthYear",  year);
     aTBContact.setValue("BirthMonth", month);
     aTBContact.setValue("BirthDay",   day);
-
-    // Anniversary
-    var anniversary = aGContact.getValue("event", 0, "anniversary");
+  },
+  /**
+   * Sets the anniversary of the given contact.
+   * @param aTBContact {TBContact} The Thunderbird contact.
+   * @param bday {string} The anniversary string.  null if not present.
+   */
+  setTBAnniversary: function ContactConverter_setTBAnniversary(aTBContact, anniversary) {
     var anniversaryYear = null, anniversaryMonth = null, anniversaryDay = null;
     if (anniversary && anniversary.value) {
       com.gContactSync.LOGGER.VERBOSE_LOG(" * Found an anniversary value of " + anniversary.value);
@@ -485,121 +693,81 @@ com.gContactSync.ContactConverter = {
     aTBContact.setValue("AnniversaryYear", anniversaryYear);
     aTBContact.setValue("AnniversaryMonth", anniversaryMonth);
     aTBContact.setValue("AnniversaryDay", anniversaryDay);
-
-    if (com.gContactSync.Preferences.mSyncPrefs.getPhotos.value) {
-
-      aTBContact = this.savePhotoFromGContact(aTBContact, aGContact);
-    }
-    
-    // Add the phonetic first and last names
-    if (com.gContactSync.Preferences.mSyncPrefs.syncPhoneticNames.value) {
-      aTBContact.setValue("PhoneticFirstName",
-                          aGContact.getAttribute("givenName",
-                          com.gContactSync.gdata.namespaces.GD.url,
-                          0,
-                          "yomi"));
-      aTBContact.setValue("PhoneticLastName",
-                          aGContact.getAttribute("familyName",
-                          com.gContactSync.gdata.namespaces.GD.url,
-                          0,
-                          "yomi"));
-    }
-
-    aTBContact.update();
-    if (ab.mPrefs.syncGroups == "true" && ab.mPrefs.myContacts != "true") {
-      // get the groups after updating the card
-      var groups = aGContact.getValue("groupMembershipInfo"),
-          lists  = com.gContactSync.Sync.mLists,
-          list,
-          group;
-      for (var i in lists) {
-        group = groups[i];
-        list  = lists[i];
-        // delete the card from the list, if necessary
-        if (list.hasContact(aTBContact)) {
-          if (!group) {
-            list.deleteContacts([aTBContact]);
-          }
-          aTBContact.update();
-        }
-        // add the card to the list, if necessary
-        else if (group) {
-          list.addContact(aTBContact);
-        }
-      }
-    }
   },
   /**
-   * Saves the photo from the given Google contact to the given TB contact if present and if it has changed
-   * since the last sync.
-   * @param aGContact {GContact} A GContact object with the photo to save.
-   * @param aTBContact {TBContact}   An existing card that can be QI'd to
-   *                            Components.interfaces.nsIAbMDBCard if this is
-   *                            before 413260 landed.  Updated with the photo from the GContact.
-   * @returns {TBContact} The updated TBContact.
+   * Sets the birthday of the given Google contact from the given TB contact.
+   * @param aGContact {GContact} The Google contact.
+   * @param aTBContact {TBContact} The Thunderbird contact to get the birthday from.
+   * @return {string} The birthday value.
    */
-  savePhotoFromGContact: function ContactConvert_savePhotoFromGContact(aTBContact, aGContact) {
-
-    var info = aGContact.getPhotoInfo();
-
-    // If the contact has a photo then save it to a local file and update
-    // the related attributes
-    // Thunderbird requires two copies of each photo.  A permanent copy must
-    // be kept outside of the Photos directory.  Each time a contact is edited
-    // Thunderbird will re-copy the original photo to the Photos directory and
-    // delete the old copy.
-
-    if (!info || !info.etag) {
-
-      // If the contact doesn't have a photo then clear the related attributes
-      aTBContact.setValue("PhotoName", "");
-      aTBContact.setValue("PhotoType", "");
-      aTBContact.setValue("PhotoURI",  "");
-      aTBContact.setValue("PhotoEtag", "");
-
-    } else if (info.etag === aTBContact.getValue("PhotoEtag")) {
-
-      com.gContactSync.LOGGER.VERBOSE_LOG(" * Photo is already up-to-date");
-
-    } else {
-
-      var file = aGContact.writePhoto(com.gContactSync.Sync.mCurrentAuthToken);
-
-      if (!file) {
-
-        com.gContactSync.LOGGER.LOG_WARNING("Failed to write contact photo");
-
+  setGoogleBirthday: function ContactConverter_setGoogleBirthday(aGContact, aTBContact) {
+    // Birthday can be either YYYY-M-D or --M-D for no year.
+    // TB can have all three, just a day/month, or just a year through the UI
+    var birthDay    = parseInt(aTBContact.getValue("BirthDay"), 10),
+        birthMonth  = (isNaN(birthDay) || (birthDay > 31)) ?
+                      NaN : parseInt(aTBContact.getValue("BirthMonth"), 10),
+        birthdayVal = null;
+    // if the contact has a birth month (and birth day) add it to the contact
+    // from Google
+    if (!isNaN(birthMonth) && (birthMonth <= 12)) {
+      var birthYear = parseInt(aTBContact.getValue("BirthYear"), 10);
+      // if the birth year is NaN or 0, use '-'
+      // otherwise pad it to 4 characters
+      if (!birthYear) {
+        birthYear = "-";
       } else {
-
-        com.gContactSync.LOGGER.VERBOSE_LOG("Wrote photo...name: " + file.leafName);
-        com.gContactSync.copyPhotoToPhotosDir(file);
-        aTBContact.setValue("PhotoName", file.leafName);
-        aTBContact.setValue("PhotoType", "file");
-        aTBContact.setValue("PhotoURI",
-                            Components.classes["@mozilla.org/network/io-service;1"]
-                                      .getService(Components.interfaces.nsIIOService)
-                                      .newFileURI(file)
-                                      .spec);
-        aTBContact.setValue("PhotoEtag", info.etag);
+        birthYear = String(birthYear);
+        while (birthYear.length < 4) {
+          birthYear = "0" + birthYear;
+        }
       }
+      // Pad the birth month to 2 characters
+      birthMonth = String(birthMonth);
+      if (birthMonth.length < 2) {
+        birthMonth = "0" + birthMonth;
+      }
+      // Pad the birth day to 2 characters
+      birthDay = String(birthDay);
+      if (birthDay.length < 2) {
+        birthDay = "0" + birthDay;
+      }
+      // form the birthday string: year-month-day
+      birthdayVal = birthYear + "-" + birthMonth + "-" + birthDay;
     }
-
-    return aTBContact;
+    com.gContactSync.LOGGER.VERBOSE_LOG(" * Birthday: " + birthdayVal);
+    aGContact.setValue("birthday", 0, null, birthdayVal);
+    return birthdayVal;
   },
   /**
-   * Check if the given string is null, of length 0, or consists only of spaces
-   * and return null if any of the listed conditions is true.
-   * This function was added to fix Bug 20389: Values with only spaces should be
-   * treated as empty
-   * @param aValue {string} The string to check.
-   * @returns null   - The string is null, of length 0, or consists only of
-                      spaces
-   *         aValue - The string has at least one character that is not a space
+   * Sets the anniversary of the given Google contact from the given TB contact.
+   * @param aGContact {GContact} The Google contact.
+   * @param aTBContact {TBContact} The Thunderbird contact to get the anniversary from.
+   * @return {string} The anniversary value.
    */
-  checkValue: function ContactConverter_checkValue(aValue) {
-    if (!aValue || !aValue.length) return null;
-    for (var i = 0; i < aValue.length; i++)
-      if (aValue[i] != " ") return aValue;
-    return null;
+  setGoogleAnniversary: function ContactConverter_setGoogleAnniversary(aGContact, aTBContact) {
+    var anniversaryDay = parseInt(aTBContact.getValue("AnniversaryDay"), 10);
+    var anniversaryMonth = (isNaN(anniversaryDay) || anniversaryDay > 31) ?
+                             NaN :
+                             parseInt(aTBContact.getValue("AnniversaryMonth"), 10);
+    var anniversaryYear = (isNaN(anniversaryMonth) || anniversaryMonth > 13) ?
+                            NaN :
+                            parseInt(aTBContact.getValue("AnniversaryYear"), 10);
+    var anniversaryVal = null;
+
+    if (anniversaryYear) {
+      anniversaryDay = String(anniversaryDay);
+      anniversaryMonth = String(anniversaryMonth);
+      if (anniversaryDay.length < 2) {
+        anniversaryDay = "0" + anniversaryDay;
+      }
+      if (anniversaryMonth.length < 2) {
+        anniversaryMonth = "0" + anniversaryMonth;
+      }
+      anniversaryVal = anniversaryYear + "-" + anniversaryMonth + "-" + anniversaryDay;
+    }
+
+    com.gContactSync.LOGGER.VERBOSE_LOG(" * Anniversary: " + anniversaryVal);
+    aGContact.setValue("event", 0, "anniversary", anniversaryVal);
+    return anniversaryVal;
   }
 };
