@@ -34,9 +34,9 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-if (!com) var com = {}; // A generic wrapper variable
+if (!com) {var com = {};} // A generic wrapper variable
 // A wrapper for all GCS functions and variables
-if (!com.gContactSync) com.gContactSync = {};
+if (!com.gContactSync) {com.gContactSync = {};}
 
 /**
  * Synchronizes a Thunderbird Address Book with Google Contacts.
@@ -47,6 +47,10 @@ com.gContactSync.Sync = {
   mContactsToDelete: [],
   /** New contacts to add to Google */
   mContactsToAdd:    [],
+  /** Contacts whose photos need to be written */
+  mContactsToUploadPhoto: [],
+  /** Contacts whose photos need to be downloaded */
+  mContactsToDownloadPhoto: [],
   /** Contacts to update */
   mContactsToUpdate: [],
   /** Groups to delete */
@@ -101,15 +105,15 @@ com.gContactSync.Sync = {
   mManualSync:    false,
   /** Stores a snapshot of the error count from synchronization up to, not including, the current AB */
   mPrevErrorCount: 0,
+  /** The access token */
+  mAccessToken: {},
+  /** The access token type */
+  mTokenType: {},
   /**
    * Performs the first steps of the sync process.
    * @param aManualSync {boolean} Set this to true if the sync was run manually.
    */
   begin: function Sync_begin(aManualSync, aAddressBooks) {
-    if (!com.gContactSync.gdata.isAuthValid()) {
-      com.gContactSync.alert(com.gContactSync.StringBundle.getStr("pleaseAuth"));
-      return;
-    }
     // quit if still syncing.
     if (com.gContactSync.Preferences.mSyncPrefs.synchronizing.value) {
       return;
@@ -143,6 +147,12 @@ com.gContactSync.Sync = {
     }
 
     com.gContactSync.Overlay.updateVersion();
+
+    if (com.gContactSync.Sync.mAddressBooks.length === 0) {
+      com.gContactSync.alert(com.gContactSync.StringBundle.getStr("pleaseAuth"));
+      return;
+    }
+
     com.gContactSync.Sync.syncNextUser();
   },
   /**
@@ -152,7 +162,7 @@ com.gContactSync.Sync = {
   syncNextUser: function Sync_syncNextUser() {
 
     // If the sync was successful, set the previous address book's last sync date (if it exists)
-    if (com.gContactSync.Sync.mPrevErrorCount == com.gContactSync.LOGGER.mErrorCount &&
+    if (com.gContactSync.Sync.mPrevErrorCount === com.gContactSync.LOGGER.mErrorCount &&
         com.gContactSync.Sync.mCurrentAb &&
         com.gContactSync.Sync.mCurrentAb.setLastSyncDate) {
       com.gContactSync.Sync.mCurrentAb.setLastSyncDate((new Date()).getTime());
@@ -188,7 +198,7 @@ com.gContactSync.Sync = {
     com.gContactSync.LOGGER.LOG("Starting synchronization for " + com.gContactSync.Sync.mCurrentUsername +
                                 " at: " + new Date().getTime() + " (" + Date() + ")\n");
     com.gContactSync.Sync.mCurrentAb        = obj.ab;
-    com.gContactSync.Sync.mCurrentAuthToken = com.gContactSync.LoginManager.getAuthTokens()[com.gContactSync.Sync.mCurrentUsername];
+    var refreshToken                        = com.gContactSync.LoginManager.getAuthTokens()[com.gContactSync.Sync.mCurrentUsername];
     com.gContactSync.Sync.mContactsUrl      = null;
     com.gContactSync.Sync.mBackup           = false;
     com.gContactSync.LOGGER.VERBOSE_LOG("Found Address Book with name: " +
@@ -203,65 +213,21 @@ com.gContactSync.Sync = {
     }
     // If an authentication token cannot be found for this username then
     // offer to let the user login with that account
-    if (!com.gContactSync.Sync.mCurrentAuthToken) {
+    if (!refreshToken) {
       com.gContactSync.LOGGER.LOG_WARNING("Unable to find the auth token for: " +
                                           com.gContactSync.Sync.mCurrentUsername);
       if (com.gContactSync.confirm(com.gContactSync.StringBundle.getStr("noTokenFound") +
                   ": " + com.gContactSync.Sync.mCurrentUsername +
                   "\n" + com.gContactSync.StringBundle.getStr("ab") +
                   ": " + com.gContactSync.Sync.mCurrentAb.getName())) {
-        // Now let the user login
-        var prompt   = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
-                                 .getService(Components.interfaces.nsIPromptService)
-                                 .promptUsernameAndPassword,
-            username = {value: com.gContactSync.Sync.mCurrentUsername},
-            password = {},
-        // opens a username/password prompt
-            ok = prompt(window, com.gContactSync.StringBundle.getStr("loginTitle"),
-                        com.gContactSync.StringBundle.getStr("loginText"), username, password, null,
-                        {value: false});
-        if (!ok) {
-          com.gContactSync.Sync.syncNextUser();
-          return;
-        }
-        // Decrement the index so Sync.syncNextUser runs on this AB again
         com.gContactSync.Sync.mIndex--;
-        // This is a primitive way of validating an e-mail address, but Google takes
-        // care of the rest.  It seems to allow getting an auth token w/ only the
-        // username, but returns an error when trying to do anything w/ that token
-        // so this makes sure it is a full e-mail address.
-        if (username.value.indexOf("@") < 1) {
-          com.gContactSync.alertError(com.gContactSync.StringBundle.getStr("invalidEmail"));
+        com.gContactSync.gdata.requestNewRefreshToken(com.gContactSync.Sync.mCurrentUsername, function newRefreshToken(aResponse) {
+          com.gContactSync.LoginManager.addAuthToken(com.gContactSync.Sync.mCurrentUsername, aResponse.refresh_token);
           com.gContactSync.Sync.syncNextUser();
-          return;
-        }
-        // fix the username before authenticating
-        username.value = com.gContactSync.fixUsername(username.value);
-        var body    = com.gContactSync.gdata.makeAuthBody(username.value, password.value);
-        var httpReq = new com.gContactSync.GHttpRequest("authenticate", null, null, body);
-        // if it succeeds and Google returns the auth token, store it and then start
-        // a new sync
-        httpReq.mOnSuccess = function reauth_onSuccess(httpReq) {
-          com.gContactSync.LoginManager.addAuthToken(username.value,
-                                                     'GoogleLogin' +
-                                                     httpReq.responseText.split("\n")[2]);
-          com.gContactSync.Sync.syncNextUser();
-        };
-        // if it fails, alert the user and prompt them to try again
-        httpReq.mOnError   = function reauth_onError(httpReq) {
-          com.gContactSync.alertError(com.gContactSync.StringBundle.getStr('authErr'));
-          com.gContactSync.LOGGER.LOG_ERROR('Authentication Error - ' +
-                                            httpReq.status,
-                                            httpReq.responseText);
-          com.gContactSync.Sync.syncNextUser();
-        };
-        // if the user is offline, alert them and quit
-        httpReq.mOnOffline = com.gContactSync.Sync.mOfflineFunction;
-        httpReq.mOn503 = com.gContactSync.Sync.m503Function;
-        httpReq.send();
-      }
-      else
+        });
+      } else {
         com.gContactSync.Sync.syncNextUser();
+      }
       return;
     }
     var lastBackup = parseInt(obj.ab.mPrefs.lastBackup, 10),
@@ -277,16 +243,38 @@ com.gContactSync.Sync = {
                                            prefix,
                                            ".bak");
     }
-    // getGroups must be called if the myContacts pref is set so it can find the
-    // proper group URL
-    if (com.gContactSync.Sync.mCurrentAb.mPrefs.syncGroups === "true" ||
-        (com.gContactSync.Sync.mCurrentAb.mPrefs.myContacts !== "false" &&
-         com.gContactSync.Sync.mCurrentAb.mPrefs.myContactsName !== "false")) {
-      com.gContactSync.Sync.getGroups();
-    }
-    else {
-      com.gContactSync.Sync.getContacts();
-    }
+    com.gContactSync.Sync.getAccessToken(refreshToken);
+  },
+  /**
+   * Exchanges the refresh token for an access token.  On success, continues synchronization.
+   *
+   * @param aRefreshToken {string} A refresh token.
+   */
+  getAccessToken: function Sync_getAccessToken(aRefreshToken) {
+    com.gContactSync.LOGGER.VERBOSE_LOG("Requesting access token");
+    // Fetch an access token from the refresh token
+    var request = new com.gContactSync.GHttpRequest("REFRESH_REQUEST", aRefreshToken);
+    request.mOnSuccess = function getAccessTokenSuccess(aHttpRequest) {
+      var response = JSON.parse(aHttpRequest.responseText);
+      com.gContactSync.Sync.mCurrentAuthToken = response.token_type + " " + response.access_token;
+      // getGroups must be called if the myContacts pref is set so it can find the
+      // proper group URL
+      if (com.gContactSync.Sync.mCurrentAb.mPrefs.syncGroups === "true" ||
+          (com.gContactSync.Sync.mCurrentAb.mPrefs.myContacts !== "false" &&
+           com.gContactSync.Sync.mCurrentAb.mPrefs.myContactsName !== "false")) {
+        com.gContactSync.Sync.getGroups();
+      }
+      else {
+        com.gContactSync.Sync.getContacts();
+      }
+    };
+    request.mOnError = function getAccessTokenError(httpReq) {
+      com.gContactSync.LOGGER.LOG_ERROR(httpReq.responseText);
+      com.gContactSync.Sync.syncNextUser(httpReq.responseText);
+    };
+    request.mOnOffline = com.gContactSync.Sync.mOfflineFunction;
+    request.mOn503 = com.gContactSync.Sync.m503Function;
+    request.send();
   },
   /**
    * Sends an HTTP Request to Google for a feed of all of the user's groups.
@@ -458,50 +446,72 @@ com.gContactSync.Sync = {
       com.gContactSync.Sync.finish("Max Contacts too low...resynchronizing", true);
       return;
     }
-    com.gContactSync.Sync.mContactsToAdd    = [];
+    com.gContactSync.Sync.mContactsToAdd = [];
+    com.gContactSync.Sync.mContactsToUploadPhoto = [];
+    com.gContactSync.Sync.mContactsToDownloadPhoto = [];
     com.gContactSync.Sync.mContactsToDelete = [];
     com.gContactSync.Sync.mContactsToUpdate = [];
     var gContact,
-     // get the strings outside of the loop so they are only found once
-        found       = " * Found a match, last modified:",
-        bothChanged = " * Conflict detected: the contact has been updated in " +
-                      "both Google and Thunderbird",
-        bothGoogle  = "  - The Google contact will be updated",
-        bothTB      = "  - The Thunderbird contact will be updated",
-        gContacts   = {};
-    // Step 1: get all contacts from Google into GContact objects in an object
+        gContacts = {},
+        gContactInfo = {},
+        abCardInfo = [];
+
+    // Get all contacts from Google into GContact objects in an object
     // keyed by ID.
     for (var i = 0, length = googleContacts.length; i < length; i++) {
       gContact               = new com.gContactSync.GContact(googleContacts[i]);
       gContact.lastModified  = gContact.getLastModifiedDate();
       gContact.id            = gContact.getID(true);
       gContacts[gContact.id] = gContact;
+      gContactInfo[gContact.id] = {name: gContact.getName(), lastModified: gContact.lastModified};
+    }
+    for (i = 0, length = abCards.length; i < length; ++i) {
+      abCardInfo.push({id: abCards[i].getID(), name: abCards[i].getName(), lastModified: abCards[i].getValue("LastModifiedDate")});
     }
     // re-initialize the contact converter (in case a pref changed)
     com.gContactSync.ContactConverter.init();
 
-    // Step 1.5: If this is the first sync then iterate through TB contacts
+    // If this is the first sync then iterate through TB contacts
     // If the contact matches a Google contact then set the TB contact's
     // GoogleID to its matching contact and LastModifiedDate to 0.
     // This prevents some duplicates on the first sync by basically overwritting
     // similar TB contacts during the first sync.
     // This is very basic, and won't merge duplicates in Thunderbird or dups
     // in Google; it just matches with the first contact it finds, if any.
-    if (lastSync == 0) {
+    if (lastSync === 0) {
       com.gContactSync.Overlay.setStatusBarText(com.gContactSync.StringBundle.getStr("firstSyncContacts"));
       for (i = 0, length = abCards.length; i < length; i++) {
-        if (abCards[i].getValue("GoogleID")) continue;
+
+        // If this address book was previously synchronized with gContactSync there's no need to merge.
+        // The contacts will conflict and the updateGoogleInConflicts pref will be used to resolve.
+        if (abCards[i].getValue("GoogleID")) {continue;}
+
         for (var id in gContacts) {
-          if (gContacts[id] && abCards[i] && !gContacts[id].merged &&
+          if (gContacts[id] && abCards[i] &&
               com.gContactSync.GAbManager.compareContacts(
                 abCards[i],
                 gContacts[id],
                 ["DisplayName", "PrimaryEmail"],
                 ["fullName",    "email"],
                 0.5)) {
-            abCards[i].setValue("GoogleID", id);
-            abCards[i].setValue("LastModifiedDate", 0);
-            gContacts[id].merged = true;
+            com.gContactSync.LOGGER.LOG(abCards[i].getName() + ": " + id);
+            com.gContactSync.LOGGER.LOG(" * Merging");
+            // Pass false to update TB during conflicts.  Other add-ons don't synchronize types so allowing Google to be updated
+            // would reset types to their default values.
+            var updated = com.gContactSync.ContactConverter.merge(abCards[i], gContacts[id], false);
+            if (updated.google) {
+              var toUpdate = {};
+              toUpdate.gContact = gContacts[id];
+              toUpdate.abCard   = abCards[i];
+              com.gContactSync.Sync.mContactsToUpdate.push(toUpdate);
+              this.mCurrentSummary.mRemote.mUpdated++;
+            }
+            if (updated.thunderbird) {
+              this.mCurrentSummary.mLocal.mUpdated++;
+            }
+            gContacts[id] = null;
+            abCards.splice(i, 1);
+            --i;
             break;
           }
         }
@@ -510,162 +520,86 @@ com.gContactSync.Sync = {
 
     com.gContactSync.Overlay.setStatusBarText(com.gContactSync.StringBundle.getStr("syncing"));
 
-    // Step 2: iterate through TB Contacts and check for matches
-    for (i = 0, length = abCards.length; i < length; i++) {
-      var tbContact  = abCards[i],
-          id         = tbContact.getID(),
-          tbCardDate = tbContact.getValue("LastModifiedDate");
-      com.gContactSync.LOGGER.LOG(tbContact.getName() + ": " + id);
-      tbContact.id = id;
-      // no ID = new contact
-      if (!id) {
-        if (ab.mPrefs.readOnly === "true") {
-          com.gContactSync.LOGGER.LOG(" * The contact is new. " +
-                                      "Ignoring since read-only mode is on.");
-          this.mCurrentSummary.mLocal.mIgnored++;
-        }
-        else {
-          // this.mCurrentSummary.mRemote.mAdded++; This is done after the contact is
-          // successfully added
-          com.gContactSync.LOGGER.LOG(" * This contact is new and will be added to Google.");
-          this.mCurrentSummary.mRemote.mAdded++;
-          com.gContactSync.Sync.mContactsToAdd.push(tbContact);
-        }
-      }
-      // if there is a matching Google Contact
-      else if (gContacts[id]) {
-        gContact   = gContacts[id];
-        // remove it from gContacts
-        gContacts[id]  = null;
-        // note that this returns 0 if readOnly is set
-        gCardDate  = ab.mPrefs.writeOnly !== "true" ? gContact.lastModified : 0; // TODO - should this be a 1?
-        // 4 options
-        // if both were updated
-        com.gContactSync.LOGGER.LOG(found +
-                                    "\n   - Google:      " + gCardDate +
-                                    " (" + new Date(gCardDate) + ")" +
-                                    "\n   - Thunderbird: " + (tbCardDate * 1000) +
-                                    " (" + new Date(tbCardDate * 1000) + ")");
-        com.gContactSync.LOGGER.VERBOSE_LOG(" * Google ID: " + id);
-        // If there is a conflict, looks at the updateGoogleInConflicts
-        // preference and updates Google if it's true, or Thunderbird if false
-        if (gCardDate > lastSync && tbCardDate > lastSync / 1000) {
-          com.gContactSync.LOGGER.LOG(bothChanged);
-          this.mCurrentSummary.mConflicted++;
-          if (ab.mPrefs.writeOnly  === "true" || ab.mPrefs.updateGoogleInConflicts === "true") {
-            com.gContactSync.LOGGER.LOG(bothGoogle);
-            var toUpdate = {};
-            toUpdate.gContact = gContact;
-            toUpdate.abCard   = tbContact;
-            this.mCurrentSummary.mRemote.mUpdated++;
-            com.gContactSync.Sync.mContactsToUpdate.push(toUpdate);
-          }
-          // update Thunderbird if writeOnly is off and updateGoogle is off
-          else {
-            com.gContactSync.LOGGER.LOG(bothTB);
-            this.mCurrentSummary.mLocal.mUpdated++;
-            com.gContactSync.ContactConverter.makeCard(gContact, tbContact);
-          }
-        }
-        // if the contact from Google is newer update the TB card
-        else if (gCardDate > lastSync) {
-          com.gContactSync.LOGGER.LOG(" * The contact from Google is newer...Updating the" +
-                                      " contact from Thunderbird");
-          this.mCurrentSummary.mLocal.mUpdated++;
-          com.gContactSync.ContactConverter.makeCard(gContact, tbContact);
-        }
-        // if the TB card is newer update Google
-        else if (tbCardDate > lastSync / 1000) {
-          com.gContactSync.LOGGER.LOG(" * The contact from Thunderbird is newer...Updating the" +
-                                      " contact from Google");
-          var toUpdate = {};
-          toUpdate.gContact = gContact;
-          toUpdate.abCard   = tbContact;
-          this.mCurrentSummary.mRemote.mUpdated++;
-          com.gContactSync.Sync.mContactsToUpdate.push(toUpdate);
-        }
-        // otherwise nothing needs to be done
-        else {
-          com.gContactSync.LOGGER.LOG(" * Neither contact has changed");
-          this.mCurrentSummary.mNotChanged++;
-        }
-      }
-      // if there isn't a match, but the card is new, add it to Google
-      else if (tbContact.getValue("LastModifiedDate") > lastSync / 1000 ||
-               isNaN(lastSync)) {
-        com.gContactSync.LOGGER.LOG(" * Contact is new, adding to Google.");
-        this.mCurrentSummary.mRemote.mAdded++;
-        com.gContactSync.Sync.mContactsToAdd.push(tbContact);
-      }
-      // Otherwise, delete the contact from the address book if writeOnly
-      // mode isn't on
-      else if (ab.mPrefs.writeOnly !== "true") {
-        com.gContactSync.LOGGER.LOG(" * Contact deleted from Google, " +
-                                    "deleting local copy");
-        this.mCurrentSummary.mLocal.mRemoved++;
-        cardsToDelete.push(tbContact);
-      } else {
-        this.mCurrentSummary.mRemote.mIgnored++;
-        com.gContactSync.LOGGER.LOG(" * Contact deleted from Google, ignoring" +
-                                    " since write-only mode is enabled");
-      }
-    }
+    var worker = new Worker("chrome://gcontactsync/content/SyncWorker.js");
+    var workerData = {
+      mABCards:                 abCardInfo,
+      mGContacts:               gContactInfo,
+      mLastSync:                lastSync,
+      mReadOnly:                ab.mPrefs.readOnly === "true",
+      mWriteOnly:               ab.mPrefs.writeOnly === "true",
+      mUpdateGoogleInConflicts: ab.mPrefs.updateGoogleInConflicts === "true",
+      mCurrentSummary:          this.mCurrentSummary,
+    };
 
-    // STEP 3: Check for old Google contacts to delete and new contacts to add to TB
-    com.gContactSync.LOGGER.LOG("**Looking for unmatched Google contacts**");
-    for (var id in gContacts) {
-      var gContact = gContacts[id];
-      if (gContact) {
-      
-        // If writeOnly is on, then set the last modified date to 1 so TB grabs
-        // all the contacts from Google during the first sync.
-        var gCardDate = ab.mPrefs.writeOnly != "true" ? gContact.lastModified : 1;
-        com.gContactSync.LOGGER.LOG(gContact.getName() + " - " + gCardDate +
-                                    "\n" + id);
-        if (gCardDate > lastSync || isNaN(lastSync)) {
-          com.gContactSync.LOGGER.LOG(" * The contact is new and will be added to Thunderbird");
-          this.mCurrentSummary.mLocal.mAdded++;
-          var newCard = ab.newContact();
-          com.gContactSync.ContactConverter.makeCard(gContact, newCard);
-        }
-        else if (ab.mPrefs.readOnly != "true") {
-          com.gContactSync.LOGGER.LOG(" * The contact is old and will be deleted");
-          this.mCurrentSummary.mLocal.mRemoved++;
-          com.gContactSync.Sync.mContactsToDelete.push(gContact);
-        }
-        else {
-          com.gContactSync.LOGGER.LOG (" * The contact was deleted in Thunderbird.  " +
-                                       "Ignoring since read-only mode is on.");
-          this.mCurrentSummary.mLocal.mIgnored++;
-        }
-      }
-    }
-    var threshold = com.gContactSync.Preferences.mSyncPrefs
-                                                .confirmDeleteThreshold.value;
-    // Request permission from the user to delete > threshold contacts from a
-    // single source
-    // If the user clicks Cancel the AB is disabled
-    if (threshold > -1 &&
-          (cardsToDelete.length >= threshold ||
-           com.gContactSync.Sync.mContactsToDelete.length >= threshold) &&
-          !com.gContactSync.Sync.requestDeletePermission(cardsToDelete.length,
-                                                         com.gContactSync.Sync.mContactsToDelete.length)) {
-        // If canceled here then reset most remote counts and local deleted to 0
-        this.mCurrentSummary.mLocal.mRemoved  = 0;
-        this.mCurrentSummary.mRemote.mAdded   = 0;
-        this.mCurrentSummary.mRemote.mRemoved = 0;
-        this.mCurrentSummary.mRemote.mUpdated = 0;
-        com.gContactSync.Sync.syncNextUser();
-        return;
-    }
-    // delete the old contacts from Thunderbird
-    if (cardsToDelete.length > 0) {
-      ab.deleteContacts(cardsToDelete);
-    }
+    // Make this == Sync when receiving messages from the worker
+    var self = this;
+    worker.onmessage = function(event) {
+      self.onworkermessage.call(self, event);
+    };
 
-    com.gContactSync.LOGGER.LOG("***Deleting contacts from Google***");
-    // delete contacts from Google
-    com.gContactSync.Sync.processDeleteQueue();
+    this.onworkermessage = function syncWorkerOnMessage(event) {
+      if (event.data.mType === "log") {
+        com.gContactSync.LOGGER.LOG(event.data.mMessage);
+      } else if (event.data.mType === "newTBContact") {
+        var newCard = ab.newContact();
+        com.gContactSync.ContactConverter.makeCard(gContacts[event.data.mID], newCard);
+        if (newCard.mUpdatePhoto) {
+          this.mContactsToDownloadPhoto.push({abCard: newCard, gContact: gContacts[event.data.mID]});
+        }
+      } else if (event.data.mType === "updateTBCard") {
+        var tbContact = abCards[event.data.mTBCardIndex];
+        com.gContactSync.ContactConverter.makeCard(gContacts[tbContact.getID()], tbContact);
+        if (tbContact.mUpdatePhoto) {
+          this.mContactsToDownloadPhoto.push({abCard: tbContact, gContact: gContacts[tbContact.getID()]});
+        }
+      } else if (event.data.mType === "done") {
+
+        for (var i in event.data.mCurrentSummary) {
+          if (event.data.mCurrentSummary.hasOwnProperty(i)) {this.mCurrentSummary[i] = event.data.mCurrentSummary[i];}
+        }
+        for (i = 0, length = event.data.mContactsToAdd.length; i < length; ++i) {
+          this.mContactsToAdd.push(abCards[event.data.mContactsToAdd[i]]);
+        }
+        for (i = 0, length = event.data.mContactsToUpdate.length; i < length; ++i) {
+          var abCard = abCards[event.data.mContactsToUpdate[i]];
+          this.mContactsToUpdate.push({abCard: abCard, gContact: gContacts[abCard.getID()]});
+        }
+        for (i = 0, length = event.data.mContactsToDelete.length; i < length; ++i) {
+          this.mContactsToDelete.push(gContacts[event.data.mContactsToDelete[i]]);
+        }
+        for (i = 0, length = event.data.mCardsToDelete.length; i < length; ++i) {
+          cardsToDelete.push(abCards[event.data.mCardsToDelete[i]]);
+        }
+
+        var threshold = com.gContactSync.Preferences.mSyncPrefs
+                                                    .confirmDeleteThreshold.value;
+        // Request permission from the user to delete > threshold contacts from a
+        // single source
+        // If the user clicks Cancel the AB is disabled
+        if (threshold > -1 &&
+              (cardsToDelete.length >= threshold ||
+               com.gContactSync.Sync.mContactsToDelete.length >= threshold) &&
+              !com.gContactSync.Sync.requestDeletePermission(cardsToDelete.length,
+                                                             com.gContactSync.Sync.mContactsToDelete.length)) {
+          // If canceled here then reset most remote counts and local deleted to 0
+          this.mCurrentSummary.mLocal.mRemoved  = 0;
+          this.mCurrentSummary.mRemote.mAdded   = 0;
+          this.mCurrentSummary.mRemote.mRemoved = 0;
+          this.mCurrentSummary.mRemote.mUpdated = 0;
+          com.gContactSync.Sync.syncNextUser();
+          return;
+        }
+        // delete the old contacts from Thunderbird
+        if (cardsToDelete.length > 0) {
+          ab.deleteContacts(cardsToDelete);
+        }
+
+        com.gContactSync.LOGGER.LOG("***Deleting contacts from Google***");
+        // delete contacts from Google
+        com.gContactSync.Sync.processDeleteQueue();
+      }
+    };
+    worker.postMessage(workerData);
   },
   /**
    * Shows a confirmation dialog asking the user to give gContactSync permission
@@ -685,12 +619,14 @@ com.gContactSync.Sync = {
     com.gContactSync.LOGGER.LOG("Requesting permission to delete " +
                                 "TB: " + aNumTB + ", Google: " + aNumGoogle +
                                 " contacts...");
+
     if (!com.gContactSync.confirm(warning)) {
       com.gContactSync.LOGGER.LOG(" * Permission denied, disabling AB");
       com.gContactSync.Sync.mCurrentAb.savePref("Disabled", true);
       com.gContactSync.alert(com.gContactSync.StringBundle.getStr("deleteCancel"));
       return false;
     }
+
     com.gContactSync.LOGGER.LOG(" * Permission granted");
     return true;
   },
@@ -802,22 +738,15 @@ com.gContactSync.Sync = {
           gcontact = new com.gContactSync.GContact(httpReq.responseXML);
       contact.setValue('GoogleID', gcontact.getID(true));
       contact.update();
-      // if photos are allowed to be uploaded to Google then do so
-      if (com.gContactSync.Preferences.mSyncPrefs.sendPhotos.value) {
-        gcontact.setPhoto(com.gContactSync.Sync.mNewPhotoURI);
-      }
-      // reset the new photo URI variable
-      com.gContactSync.Sync.mNewPhotoURI = null;
-      // NOTE - Google has an undocumented limit on the rate at which photos
-      // can be added.  Add a small wait here.
+      // if photos are allowed to be uploaded to Google then queue the request
       if (com.gContactSync.Preferences.mSyncPrefs.sendPhotos.value &&
-          com.gContactSync.Preferences.mSyncPrefs.newContactPhotoDelay.value > 0) {
-        setTimeout(com.gContactSync.Sync.processAddQueue,
-                   com.gContactSync.Preferences.mSyncPrefs.newContactPhotoDelay.value);
-      } else {
-        com.gContactSync.Sync.delayedProcessQueue(com.gContactSync.Sync.processAddQueue);
+          com.gContactSync.Sync.mNewPhotoURI) {
+        com.gContactSync.Sync.mContactsToUploadPhoto.push({abCard: com.gContactSync.ContactConverter.mCurrentCard,
+                                                          gContact: gcontact,
+                                                          uri: com.gContactSync.Sync.mNewPhotoURI});
       }
-    }
+      com.gContactSync.Sync.delayedProcessQueue(com.gContactSync.Sync.processAddQueue);
+    };
     httpReq.mOnCreated = onCreated;
     httpReq.mOnError   = function contactCreatedError(httpReq) {
       com.gContactSync.LOGGER.LOG_ERROR('Error while adding contact',
@@ -830,26 +759,21 @@ com.gContactSync.Sync = {
   },
   /**
    * Updates all cards to Google included in the mContactsToUpdate array one at
-   * a time to avoid timing conflicts.  Calls
-   * com.gContactSync.Sync.syncNextUser() when done.
+   * a time to avoid timing conflicts.
    */
   processUpdateQueue: function Sync_processUpdateQueue() {
+
     var ab = com.gContactSync.Sync.mCurrentAb;
-    if (!com.gContactSync.Sync.mContactsToUpdate
-        || com.gContactSync.Sync.mContactsToUpdate.length == 0
-        || ab.mPrefs.readOnly == "true") {
-      if (com.gContactSync.Sync.mAddressBooks[com.gContactSync.Sync.mIndex]) {
-        var delay = com.gContactSync.Preferences.mSyncPrefs.accountDelay.value;
-        com.gContactSync.LOGGER.LOG("**About to wait " + delay +
-                                    " ms before synchronizing the next account**");
-        com.gContactSync.Overlay.setStatusBarText(com.gContactSync.StringBundle.getStr("waiting"));
-        setTimeout(com.gContactSync.Sync.syncNextUser, delay);
-      }
-      else {
-        com.gContactSync.Sync.syncNextUser();
-      }
+
+    if (!com.gContactSync.Sync.mContactsToUpdate ||
+        (com.gContactSync.Sync.mContactsToUpdate.length === 0) ||
+        (ab.mPrefs.readOnly === "true")) {
+
+      com.gContactSync.LOGGER.LOG("***Uploading contact photos to Google***");
+      com.gContactSync.Sync.processUpdatePhotoQueue();
       return;
     }
+
     com.gContactSync.Overlay.setStatusBarText(com.gContactSync.StringBundle.getStr("updating") + " " +
                                               com.gContactSync.Sync.mContactsToUpdate.length + " " +
                                               com.gContactSync.StringBundle.getStr("remaining"));
@@ -867,8 +791,11 @@ com.gContactSync.Sync = {
     }
 
     var string = com.gContactSync.serialize(gContact.xml);
-    if (com.gContactSync.Preferences.mSyncPrefs.verboseLog.value)
+    if (com.gContactSync.Preferences.mSyncPrefs.verboseLog.value) {
       com.gContactSync.LOGGER.LOG(" * XML of contact being updated:\n" + string + "\n");
+    }
+    com.gContactSync.Sync.mNewPhotoURI = com.gContactSync.Preferences.mSyncPrefs.sendPhotos.value ?
+                                         gContact.mNewPhotoURI : null;
     var httpReq = new com.gContactSync.GHttpRequest("update",
                                                     com.gContactSync.Sync.mCurrentAuthToken,
                                                     editURL,
@@ -876,6 +803,16 @@ com.gContactSync.Sync = {
                                                     com.gContactSync.Sync.mCurrentUsername);
     httpReq.addHeaderItem("If-Match", "*");
     httpReq.mOnSuccess = function processUpdateSuccess(httpReq) {
+
+      // if photos are allowed to be uploaded to Google then queue the request
+      if (com.gContactSync.Preferences.mSyncPrefs.sendPhotos.value &&
+          com.gContactSync.Sync.mNewPhotoURI) {
+
+        var gcontact = new com.gContactSync.GContact(httpReq.responseXML);
+        com.gContactSync.Sync.mContactsToUploadPhoto.push({abCard: com.gContactSync.ContactConverter.mCurrentCard,
+                                                          gContact: gcontact,
+                                                          uri: com.gContactSync.Sync.mNewPhotoURI});
+      }
       com.gContactSync.Sync.delayedProcessQueue(com.gContactSync.Sync.processUpdateQueue);
     };
     httpReq.mOnError   = function processUpdateError(httpReq) {
@@ -887,6 +824,83 @@ com.gContactSync.Sync = {
     httpReq.mOn503 = com.gContactSync.Sync.m503Function;
     httpReq.send();
   },
+  /**
+   * Uploads new and updated photos to Google.
+   * Calls com.gContactSync.Sync.processDownloadPhotoQueue() when done.
+   */
+  processUpdatePhotoQueue: function Sync_processUpdatePhotoQueue() {
+
+    var ab = com.gContactSync.Sync.mCurrentAb;
+
+    if (!com.gContactSync.Sync.mContactsToUploadPhoto ||
+        (com.gContactSync.Sync.mContactsToUploadPhoto.length === 0) ||
+        (ab.mPrefs.readOnly === "true")) {
+
+      com.gContactSync.LOGGER.LOG("***Download contact photos from Google***");
+      com.gContactSync.Sync.processDownloadPhotoQueue();
+      return;
+    }
+
+    com.gContactSync.Overlay.setStatusBarText(com.gContactSync.StringBundle.getStr("uploadingPhotos") + " " +
+                                              com.gContactSync.Sync.mContactsToUploadPhoto.length + " " +
+                                              com.gContactSync.StringBundle.getStr("remaining"));
+    var obj = com.gContactSync.Sync.mContactsToUploadPhoto.shift();
+    com.gContactSync.LOGGER.LOG("\n" + obj.abCard.getName());
+    obj.gContact.uploadPhoto(obj.uri);
+    com.gContactSync.Sync.delayedProcessQueue(com.gContactSync.Sync.processUpdatePhotoQueue);
+  },
+  /**
+   * Downloads new photos from Google.
+   * Calls com.gContactSync.Sync.syncNextUser() when done.
+   */
+  processDownloadPhotoQueue: function Sync_processDownloadPhotoQueue() {
+
+    var ab = com.gContactSync.Sync.mCurrentAb;
+
+    if (!com.gContactSync.Sync.mContactsToDownloadPhoto ||
+        (com.gContactSync.Sync.mContactsToDownloadPhoto.length === 0) ||
+        (ab.mPrefs.readOnly === "true")) {
+
+      if (com.gContactSync.Sync.mAddressBooks[com.gContactSync.Sync.mIndex]) {
+        var delay = com.gContactSync.Preferences.mSyncPrefs.accountDelay.value;
+        com.gContactSync.LOGGER.LOG("**About to wait " + delay +
+                                    " ms before synchronizing the next account**");
+        com.gContactSync.Overlay.setStatusBarText(com.gContactSync.StringBundle.getStr("waiting"));
+        setTimeout(com.gContactSync.Sync.syncNextUser, delay);
+      } else {
+        com.gContactSync.Sync.syncNextUser();
+      }
+      return;
+    }
+
+    com.gContactSync.Overlay.setStatusBarText(com.gContactSync.StringBundle.getStr("downloadingPhotos") + " " +
+                                              com.gContactSync.Sync.mContactsToDownloadPhoto.length + " " +
+                                              com.gContactSync.StringBundle.getStr("remaining"));
+
+    var obj = com.gContactSync.Sync.mContactsToDownloadPhoto.shift();
+    com.gContactSync.LOGGER.LOG("\n" + obj.abCard.getName());
+    var file = obj.gContact.writePhoto(com.gContactSync.Sync.mCurrentAuthToken);
+
+    if (!file) {
+
+      com.gContactSync.LOGGER.LOG_WARNING("Failed to write contact photo");
+
+    } else {
+
+      com.gContactSync.LOGGER.VERBOSE_LOG("Wrote photo...name: " + file.leafName);
+      com.gContactSync.copyPhotoToPhotosDir(file);
+      obj.abCard.setValue("PhotoName", file.leafName);
+      obj.abCard.setValue("PhotoType", "file");
+      obj.abCard.setValue("PhotoURI",
+                          Components.classes["@mozilla.org/network/io-service;1"]
+                                    .getService(Components.interfaces.nsIIOService)
+                                    .newFileURI(file)
+                                    .spec);
+      obj.abCard.update();
+    }
+
+    com.gContactSync.Sync.delayedProcessQueue(com.gContactSync.Sync.processDownloadPhotoQueue);
+  },
   /**
    * Syncs all contact groups with mailing lists.
    * @param aAtom {XML} The ATOM/XML feed of Groups.
