@@ -15,7 +15,7 @@
  *
  * The Initial Developer of the Original Code is
  * Josh Geenen <gcontactsync@pirules.org>.
- * Portions created by the Initial Developer are Copyright (C) 2008-2009
+ * Portions created by the Initial Developer are Copyright (C) 2008-2015
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
@@ -34,13 +34,14 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-if (!com) var com = {}; // A generic wrapper variable
+if (!com) {var com = {};} // A generic wrapper variable
 // A wrapper for all GCS functions and variables
-if (!com.gContactSync) com.gContactSync = {};
+if (!com.gContactSync) {com.gContactSync = {};}
 
 window.addEventListener("load",
   /** Initializes the gdata class when the window has finished loading */
-  function gCS_gdataLoadListener(e) {
+  function gCS_gdataLoadListener() {
+    window.removeEventListener("load", gCS_gdataLoadListener, false);
     com.gContactSync.gdata.contacts.init();
   },
 false);
@@ -52,31 +53,38 @@ false);
  * @class
  */
 com.gContactSync.gdata = {
-  AUTH_URL:              "https://www.google.com/accounts/ClientLogin",
-  AUTH_REQUEST_TYPE:     "POST",
-  AUTH_SUB_SESSION_URL:  "https://www.google.com/accounts/AuthSubSessionToken",
-  AUTH_SUB_SESSION_TYPE: "GET",
-  AUTH_SUB_REVOKE_URL:   "https://www.google.com/accounts/AuthSubRevokeToken",
-  AUTH_SUB_REVOKE_TYPE:  "GET",
-  O_AUTH_URL:            "https://www.google.com/accounts/AuthSubRequest?scope=https%3A%" +
-                         "2F%2Fwww.google.com%2Fm8%2Ffeeds%2F&session=1&secure=0&next=" +
-                         "http%3A%2F%2Fpirules.org%2Ftools%2Fgcs%2Findex.php",
-  O_AUTH_TYPE:           "GET",
+  CLIENT_ID:                  "874495714229-5m7jmsjebv6nrf61q14siutq43bi1gvt.apps.googleusercontent.com",
+  CLIENT_SECRET:              "vayAK3lt9f4tMcMK1HZ4XZqG",
+  REDIRECT_URI:               "urn:ietf:wg:oauth:2.0:oob",
+  REDIRECT_TITLE:             "Success code=",
+  RESPONSE_TYPE:              "code",
+  SCOPE:                      "https://www.google.com/m8/feeds",
+  OAUTH_URL:                  "https://accounts.google.com/o/oauth2/auth",
+  TOKEN_REQUEST_URL:          "https://accounts.google.com/o/oauth2/token",
+  TOKEN_REQUEST_TYPE:         "POST",
+  TOKEN_REQUEST_GRANT_TYPE:   "authorization_code",
+  REFRESH_REQUEST_URL:        "https://accounts.google.com/o/oauth2/token",
+  REFRESH_REQUEST_TYPE:       "POST",
+  REFRESH_REQUEST_GRANT_TYPE: "refresh_token",
+  AUTH_URL:                   "https://www.google.com/accounts/ClientLogin",
+  AUTH_REQUEST_TYPE:          "POST",
+  AUTH_SUB_SESSION_URL:       "https://www.google.com/accounts/AuthSubSessionToken",
+  AUTH_SUB_SESSION_TYPE:      "GET",
+  AUTH_SUB_REVOKE_URL:        "https://www.google.com/accounts/AuthSubRevokeToken",
+  AUTH_SUB_REVOKE_TYPE:       "GET",
   /**
-   * Sets up the body for an authentication request given the e-mail address
-   * and password.
-   * @param aEmail     {string} The user's e-mail address
-   * @param aPassword  {string} The user's password
-   * @returns {string} The body for an authentication request.
+   * Returns an OAuth URL for the given e-mail addres.
+   *
+   * @param aEmail {string} The e-mail address.
+   * @return {string} The OAuth URL.
    */
-  makeAuthBody: function gdata_makeAuthBody(aEmail, aPassword) {
-    // NOTE: leave accountType as HOSTED_OR_GOOGLE or Google Apps for your
-    // domain accounts won't work
-    // fix the username (remove whitespace)
-    aEmail = com.gContactSync.fixUsername(aEmail);
-    return "accountType=HOSTED_OR_GOOGLE&Email=" + encodeURIComponent(aEmail) +
-           "&Passwd=" + encodeURIComponent(aPassword) +
-           "&service=cp&source=Josh-gContactSync-0-3";
+  getOAuthURL: function gdata_getOAuthURL(aEmail) {
+    return com.gContactSync.gdata.OAUTH_URL +
+           "?response_type=" + com.gContactSync.gdata.RESPONSE_TYPE +
+           "&client_id=" + com.gContactSync.gdata.CLIENT_ID +
+           "&redirect_uri=" + com.gContactSync.gdata.REDIRECT_URI +
+           "&scope=" + com.gContactSync.gdata.SCOPE +
+           "&login_hint=" + aEmail;
   },
   /**
    * Returns the email address of the given ID.
@@ -270,6 +278,9 @@ com.gContactSync.gdata = {
         case "website":
           arr = this.WEBSITE_TYPES;
           break;
+        case "event":
+          arr = this.EVENT_TYPES;
+          break;
         default:
           throw "Unrecognized tagName '" + aElement.tagName + "' in setRelOrLabel";
       }
@@ -324,6 +335,9 @@ com.gContactSync.gdata = {
     POSTAL_ADDRESS_TYPES: [
       "home", "work", "other"
     ],
+    EVENT_TYPES: [
+      "other", "anniversary"
+    ],
     /** Tags that are valid an an organization tag */
     ORG_TAGS: {
       orgDepartment:     "1",
@@ -443,5 +457,30 @@ com.gContactSync.gdata = {
     com.gContactSync.LOGGER.LOG("Beginning a backup of the Google Account:\n" +
                                 aAccount + "\nto:\n" + destFile.path);
     return com.gContactSync.FileIO.writeToFile(destFile, aFeed);
-  }
+  },
+  /**
+   * Requests a new refresh token.
+   * @param aEmail {string} The email address to request a token for.
+   * @param aCallback {function} The function to call once there is a new token.
+   */
+  requestNewRefreshToken: function gdata_requestNewRefreshToken(aEmail, aCallback) {
+    var wizard = window.open("chrome://gcontactsync/content/NewRefreshToken.xul",
+                             "NewRefreshTokenWindow",
+                             "chrome,resizable=yes,scrollbars=no,status=no");
+    // when the setup window loads, set its onunload property to begin a sync
+    wizard.addEventListener("load", function onloadListener() {
+      var browser = wizard.document.getElementById("browser");
+      var url = com.gContactSync.gdata.getOAuthURL(aEmail);
+      com.gContactSync.LOGGER.VERBOSE_LOG("Opening browser with URL: " + url);
+      browser.loadURI(url);
+      com.gContactSync.OAuth2.init(browser,
+                                   com.gContactSync.gdata.REDIRECT_URI,
+                                   function callback(aResponse) {
+                                      wizard.close();
+                                      aCallback(aResponse);
+                                    },
+                                   com.gContactSync.gdata.REDIRECT_TITLE
+      );
+    });
+  },
 };
