@@ -15,7 +15,7 @@
  *
  * The Initial Developer of the Original Code is
  * Josh Geenen <gcontactsync@pirules.org>.
- * Portions created by the Initial Developer are Copyright (C) 2008-2014
+ * Portions created by the Initial Developer are Copyright (C) 2008-2015
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
@@ -45,11 +45,11 @@ if (!com.gContactSync) {
 }
 
 /** The major version of gContactSync (ie 0 in 0.2.18) */
-com.gContactSync.versionMajor   = "1";
+com.gContactSync.versionMajor   = "2";
 /** The minor version of gContactSync (ie 3 in 0.3.0b1) */
 com.gContactSync.versionMinor   = "0";
 /** The release for the current version of gContactSync (ie 1 in 0.3.1a7) */
-com.gContactSync.versionRelease = "0";
+com.gContactSync.versionRelease = "5";
 /** The suffix for the current version of gContactSync (ie a7 for Alpha 7) */
 com.gContactSync.versionSuffix  = "";
 /** The attribute where the dummy e-mail address is stored */
@@ -677,3 +677,17 @@ com.gContactSync.version04Upgrade = function gCS_version04Upgrade() {
   com.gContactSync.Preferences.setSyncPref("v04RCUpgradeNeeded", false);
 };
 
+/**
+ * Parses the parameters in the given URL and returns them in a map.
+ *
+ * @param aURL {string} The URL to parse.
+ */
+com.gContactSync.parseURLParameters = function gCS_parseURLParameters(aURL) {
+  var ret = {};
+  var params = aURL.split("?")[1].split("&");
+  for (var i in params) {
+    var param = params[i].split("=");
+    ret[param[0]] = param[1];
+  }
+  return ret;
+};
